package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public class EnclosingMethod extends Attribute
{
    private int classIndex;
    private int methodIndex;
    
    EnclosingMethod(final int nameIndex, final int len, final DataInput input, final ConstantPool cpool) throws IOException {
        this(nameIndex, len, input.readUnsignedShort(), input.readUnsignedShort(), cpool);
    }
    
    private EnclosingMethod(final int nameIndex, final int len, final int classIdx, final int methodIdx, final ConstantPool cpool) {
        super((byte)18, nameIndex, len, cpool);
        this.classIndex = classIdx;
        this.methodIndex = methodIdx;
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitEnclosingMethod(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        return (Attribute)this.clone();
    }
    
    @Override
    public final void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.classIndex);
        file.writeShort(this.methodIndex);
    }
    
    public final ConstantClass getEnclosingClass() {
        return super.getConstantPool().getConstant(this.classIndex, (byte)7, ConstantClass.class);
    }
    
    public final int getEnclosingClassIndex() {
        return this.classIndex;
    }
    
    public final ConstantNameAndType getEnclosingMethod() {
        if (this.methodIndex == 0) {
            return null;
        }
        return super.getConstantPool().getConstant(this.methodIndex, (byte)12, ConstantNameAndType.class);
    }
    
    public final int getEnclosingMethodIndex() {
        return this.methodIndex;
    }
    
    public final void setEnclosingClassIndex(final int idx) {
        this.classIndex = idx;
    }
    
    public final void setEnclosingMethodIndex(final int idx) {
        this.methodIndex = idx;
    }
}
